import click

from sdi_checker.libs.audits import Audits
from sdi_checker.app_cli import display_audits
# from sdi_checker.app_cli import display


@click.command(name='list')
@click.argument('search', nargs=-1)
@click.option('--id', '-i', default=[], multiple=True, help='List of id to limit result list of items')
@click.option('--limit', '-l', default='10', type=str, help='Number of items to return')
@click.option('--page', '-p', default=None, type=str, help='Returned paged list - format: "page,nb_items"')
@click.option('--orderby', '-o', default=['-date_modified'], multiple=True, help='Order list items result')
@click.option('--export', '-e', default=None, help='File to export result list')
@click.option('--csv', '-c', default='', multiple=False, type=str, help='Filename to save data to CSV')
@click.pass_obj
def run(app, search, id, limit, page, orderby, export, csv):
    """
    > master list [SEARCH] [--limit LIMIT] [--orderby ORDER/-ORDER] [--export FILE]
    > master list 'test + DONE:' '#tag1 | #tag2' --limit 5,3 [--page 2,20] [--orderby -title] [--export FILE]
    """
    on_list(app, search, id, limit, page, orderby, export, csv)


def on_list(app, search=None, id=None, limit=None, page=None, orderby=['-date_modified'], export=None, csv=None):
    search = ' '.join(search) if search is not None else None

    audits = Audits(file=app.audits_file)
    audits_list = audits.get(search=search).result
    result = display_audits.print_audits_list(app, audits_list, title="Liste des audits")
    
    if csv:
        # TODO: A vérifier
        app.save_data_to_csv(csv, [audits])

    if export:
        # TODO: A vérifier
        with open(export, 'w') as f:
            result_text = '\n'.join(result)
            f.write(result_text)
    
    result_text = '\n'.join(result)
    app.echo(result_text)
    

    