import click

from sdi_checker.app_cli import display_report


@click.command(name='reports')
@click.argument('files', nargs=-1, default=None)
@click.option('--directory', '-d', default='', multiple=False, type=str, help='Path to directory of report files')
@click.option('--directory', '-d', default='', multiple=False, type=str, help='Path to directory of report files')
@click.option('--csv', '-c', default='', multiple=False, type=str, help='Filename to save data to CSV')
@click.option('--export', '-e', default=None, type=str, help='Filename to export result')
@click.pass_obj
def run(app, files, directory, csv, export):
    """
    > reports [FILES] [--csv CVS] [--export EXPORT]

    Affiche la liste des fichiers disponibles ou les informations du ou des fichiers indiqués par FILES
    """
    on_reports(app, files, directory, csv, export)


def on_reports(app, files, directory, csv, export):
    """
    Display files report list
    > report reports [FILES]  
    """
    result = []
    reports = app.config['reports']
    files = [int(i.strip()) for i in ','.join(files).split(',')] if files and files is not None and len(files) > 0 else files
    
    if files and files is not None:
        for file in files:
            if file < len(reports):
                data = app.get_report_summary(report=reports[file])
                result.extend(display_report.print_report_summary(app, report=reports[file], data=data))
            else:
                result.append('')
                result.append('ERROR: file indice {file} doesn''t exist'.format(file=file))
    else:
        result.extend(display_report.print_reports_list(app, reports, files, title="Liste des fichiers"))
   
    if csv:
        app.save_data_to_csv(csv, [data])
        
    if export:
        with open(export, 'w') as f:
            result_text = '\n'.join(result)
            f.write(result_text)
    
    result_text = '\n'.join(result)
    app.echo(result_text)
