import click

from sdi_checker.app_cli import display_report


@click.command(name='ws')
@click.argument('file', nargs=-1)
@click.option('--directory', '-d', default='', multiple=False, type=str, help='Path to directory of report files')
@click.option('--csv', '-c', default='', multiple=False, type=str, help='Filename to save data to CSV')
@click.option('--search', '-s', default='', multiple=False, type=str, help='Search query to filter workspaces list')
@click.option('--limit', '-l', default='10', type=str, help='Number of workspaces to return')
@click.option('--export', '-e', default=None, type=str, help='Filename to export result')
@click.pass_obj
def run(app, file, directory, csv, search, limit, export):
    """
    > ws [FILE] [--csv CSV] [--search SEARCH] [--limit LIMIT] [--export EXPORT]
    Affiche la liste des workspaces du rapport [FILE]
    """
    on_workspaces(app, file, directory, csv, search, limit, export)


def on_workspaces(app, file, directory, csv, search, limit, export):
    """
    > ws [FILE] [--csv CSV] [--search SEARCH] [--limit LIMIT] [--export EXPORT]
    Affiche la liste des workspaces du rapport [FILE]
    """

    if not file or file is None or len(file) == 0:
        app.echo()
        app.echo('ERROR: [FILE] argument is missing.')
        display_report.print_reports_list(app, app.config['reports'], title="Thanks to indicate a [FILE] id.", echo=True)
        sys.exit()
        
    # TODO: check if file is integer (list of reports) or string (path or URL - start with 'http')
    file = [int(i.strip()) for i in ','.join(file).split(',')][0]
    report = app.config['reports'][file]
    
    if report['type'].lower() not in ['wms', 'wfs']:
        app.echo()
        app.echo('ERROR: "ws" command works only on WMS and WFS report type.')
        app.echo()
        sys.exit()
    
    result = []
    data = app.get_workspaces(report=report, filter=search)
    result.extend(display_report.print_workspaces(app, report, data, limit, search))

    if csv:
        app.save_data_to_csv(csv, data['workspaces'])

    if export:
        with open(export, 'w') as f:
            result_text = '\n'.join(result)
            f.write(result_text)
                
    result_text = '\n'.join(result)
    app.echo(result_text)
